// based on https://github.com/projecthorus/wenet/blob/master/tx/ldpc_enc.c

use bitvec::{prelude::Msb0, view::BitView};

use crate::hrow::H_ROW_2064;

const PARITY_BIT_LEN: usize = 516;
const NWT: usize = 12;

// overwrites the last 65 bytes with the checksum
pub fn ldpc_encode(buf: &mut [u8; 256 + 2 + 65]) {
    let (data, parity) = buf.split_at_mut(258);

    let data_bits = data.view_bits::<Msb0>();
    let parity_bits = parity.view_bits_mut::<Msb0>();

    let mut prev = 0;
    for p in 0..PARITY_BIT_LEN {
        let mut par = 0;
        for i in 0..NWT {
            // -1 because HROW is from Matlab, where arrays start at 1
            if data_bits[H_ROW_2064[p * NWT + i] - 1] {
                par += 1;
            }
        }

        prev = (par + prev) & 1;
        *parity_bits.get_mut(p).unwrap() = prev == 1;
    }
}
